#ifndef MODE_H
#define MODE_H

#include "appwindow.h"
#include "savable.h"

#include <QString>
#include <QWidget>
#include <QButtonGroup>
#include <QToolBar>
#include <QDockWidget>
#include <set>

class Mode : public QObject, public Savable
{
    Q_OBJECT
public:
    Mode(AppWindow *window, QString name);

    virtual void activate(); // derived classes must call Mode::activate before doing anything
    virtual void deactivate(); // derived classes must call Mode::deactivate before returning
    virtual void shutdown(){}; // called when the application is about to exit
    QString getName() const;
    static Mode *getActiveMode();

    virtual void initializeDevice() = 0;
    virtual void deviceDisconnected(){};

    virtual void saveSreenshot();
signals:
    void statusbarMessage(QString msg);
protected:
    void setStatusbarMessage(QString msg);
    // call once the derived class is fully initialized
    void finalize(QWidget *centralWidget);
    AppWindow *window;
    std::set<QAction*> actions;
    std::set<QToolBar*> toolbars;
    std::set<QDockWidget*> docks;

private:
    static Mode *activeMode;
    static QWidget *cornerWidget;
    static QButtonGroup *modeButtonGroup;
    const QString name;
    QString statusbarMsg;
    QWidget *central;
};

#endif // MODE_H
